@@ -33,6 +33,7 @@
 struct _IdeEditorWorkbenchAddin
 {
   GObject               parent_instance;
+  PnlDockManager       *manager;
   IdeEditorPerspective *perspective;
   IdeWorkbench         *workbench;
 };
@@ -65,7 +66,10 @@ ide_editor_workbench_addin_load (IdeWorkbenchAddin *addin,
 
   self->workbench = workbench;
 
+  self->manager = pnl_dock_manager_new ();
+
   self->perspective = g_object_new (IDE_TYPE_EDITOR_PERSPECTIVE,
+                                    "manager", self->manager,
                                     "visible", TRUE,
                                     NULL);
   ide_workbench_add_perspective (workbench, IDE_PERSPECTIVE (self->perspective));
@@ -90,6 +94,8 @@ ide_editor_workbench_addin_unload (IdeWorkbenchAddin *addin,
   self->workbench = NULL;
 
   ide_workbench_remove_perspective (workbench, perspective);
+
+  g_clear_object (&self->manager);
 }
 
 static gboolean
@@ -173,11 +179,10 @@ ide_editor_workbench_addin_open_cb (GObject      *object,
 
       if (sscanf (fragment, "L%u_%u", &line, &column) >= 1)
         {
-          GtkTextIter iter;
+          g_autoptr(IdeSourceLocation) location = NULL;
 
-          /* Reminder this is only safe on 3.20 */
-          gtk_text_buffer_get_iter_at_line_offset (GTK_TEXT_BUFFER (buffer), &iter, line, column);
-          gtk_text_buffer_select_range (GTK_TEXT_BUFFER (buffer), &iter, &iter);
+          location = ide_source_location_new (ide_buffer_get_file (buffer), line, column, 0);
+          ide_editor_perspective_focus_location (self->perspective, location);
         }
     }
 
