@@ -165,6 +165,30 @@ ide_application_plugins_enabled_changed (IdeApplication *self,
     peas_engine_unload_plugin (engine, plugin_info);
 }
 
+static GSettings *
+_ide_application_plugin_get_settings (IdeApplication *self,
+                                      const gchar    *module_name)
+{
+  GSettings *settings;
+
+  if (G_UNLIKELY(self->plugin_settings == NULL))
+    {
+      self->plugin_settings = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                                     g_free, g_object_unref);
+    }
+
+  if (!(settings = g_hash_table_lookup (self->plugin_settings, module_name)))
+    {
+      g_autofree gchar *path = NULL;
+
+      path = g_strdup_printf ("/org/gnome/builder/plugins/%s/", module_name);
+      settings = g_settings_new_with_path ("org.gnome.builder.plugin", path);
+      g_hash_table_insert (self->plugin_settings, g_strdup (module_name), settings);
+    }
+
+  return settings;
+}
+
 void
 ide_application_load_plugins (IdeApplication *self)
 {
@@ -180,12 +204,10 @@ ide_application_load_plugins (IdeApplication *self)
     {
       PeasPluginInfo *plugin_info = list->data;
       GSettings *settings;
-      g_autofree gchar *path = NULL;
       const gchar *module_name;
 
       module_name = peas_plugin_info_get_module_name (plugin_info);
-      path = g_strdup_printf ("/org/gnome/builder/plugins/%s/", module_name);
-      settings = g_settings_new_with_path ("org.gnome.builder.plugin", path);
+      settings = _ide_application_plugin_get_settings (self, module_name);
 
       g_object_set_data (G_OBJECT (settings), "PEAS_PLUGIN_INFO", plugin_info);
 
@@ -362,6 +384,9 @@ ide_application_unload_plugin_css (IdeApplication *self,
   g_assert (plugin_info != NULL);
   g_assert (PEAS_IS_ENGINE (engine));
 
+  if (self->plugin_css == NULL)
+    self->plugin_css = g_hash_table_new_full (NULL, NULL, NULL, g_object_unref);
+
   provider = g_hash_table_lookup (self->plugin_css, plugin_info);
 
   if (provider != NULL)
@@ -410,5 +435,16 @@ ide_application_init_plugin_menus (IdeApplication *self)
   list = peas_engine_get_plugin_list (engine);
 
   for (; list != NULL; list = list->next)
-    ide_application_load_plugin_menus (self, list->data, engine);
+    {
+      PeasPluginInfo *plugin_info = list->data;
+      const gchar *module_name;
+      GSettings *settings;
+
+      module_name = peas_plugin_info_get_module_name (plugin_info);
+      settings = _ide_application_plugin_get_settings (self, module_name);
+      if (!g_settings_get_boolean (settings, "enabled"))
+        continue;
+
+      ide_application_load_plugin_menus (self, list->data, engine);
+    }
 }
