@@ -54,6 +54,8 @@ G_DEFINE_TYPE (IdeApplication, ide_application, GTK_TYPE_APPLICATION)
 static void
 ide_application_make_skeleton_dirs (IdeApplication *self)
 {
+  g_autoptr(GSettings) settings = NULL;
+  g_autofree gchar *projects_dir = NULL;
   gchar *path;
 
   IDE_ENTRY;
@@ -72,6 +74,19 @@ ide_application_make_skeleton_dirs (IdeApplication *self)
   g_mkdir_with_parents (path, 0750);
   g_free (path);
 
+  settings = g_settings_new ("org.gnome.builder");
+  projects_dir = g_settings_get_string (settings, "projects-directory");
+
+  if (!g_path_is_absolute (projects_dir))
+    {
+      g_autofree gchar *tmp = projects_dir;
+
+      projects_dir = g_build_filename (g_get_home_dir (), tmp, NULL);
+    }
+
+  if (!g_file_test (projects_dir, G_FILE_TEST_IS_DIR))
+    g_mkdir_with_parents (projects_dir, 0750);
+
   IDE_EXIT;
 }
 
@@ -137,14 +152,22 @@ ide_application_register_menus (IdeApplication *self)
 static void
 ide_application_register_search_paths (IdeApplication *self)
 {
+  GtkSourceStyleSchemeManager *manager;
+  g_autofree gchar *gedit_path = NULL;
+
   g_assert (IDE_IS_APPLICATION (self));
 
-  gtk_source_style_scheme_manager_append_search_path (gtk_source_style_scheme_manager_get_default (),
+  manager = gtk_source_style_scheme_manager_get_default ();
+
+  gtk_source_style_scheme_manager_append_search_path (manager,
                                                       PACKAGE_DATADIR"/gtksourceview-3.0/styles/");
 
+  /* We can use styles from gedit too */
+  gedit_path = g_build_filename (g_get_user_data_dir (), "gedit", "styles", NULL);
+  gtk_source_style_scheme_manager_append_search_path (manager, gedit_path);
+
   if (g_getenv ("GB_IN_TREE_STYLE_SCHEMES") != NULL)
-    gtk_source_style_scheme_manager_prepend_search_path (gtk_source_style_scheme_manager_get_default (),
-                                                         SRCDIR"/data/style-schemes");
+    gtk_source_style_scheme_manager_prepend_search_path (manager, SRCDIR"/data/style-schemes");
 }
 
 static void
@@ -361,6 +384,18 @@ ide_application_open (GApplication  *application,
 
 }
 
+static void
+ide_application_shutdown (GApplication *application)
+{
+  IdeApplication *self = (IdeApplication *)application;
+
+  if (self->worker_manager != NULL)
+    ide_worker_manager_shutdown (self->worker_manager);
+
+  if (G_APPLICATION_CLASS (ide_application_parent_class)->shutdown)
+    G_APPLICATION_CLASS (ide_application_parent_class)->shutdown (application);
+}
+
 static void
 ide_application_finalize (GObject *object)
 {
@@ -394,6 +429,7 @@ ide_application_class_init (IdeApplicationClass *klass)
   g_app_class->local_command_line = ide_application_local_command_line;
   g_app_class->open = ide_application_open;
   g_app_class->startup = ide_application_startup;
+  g_app_class->shutdown = ide_application_shutdown;
 }
 
 static void
