@@ -34,6 +34,7 @@
 #include "ide-extension-adapter.h"
 #include "ide-file.h"
 #include "ide-file-settings.h"
+#include "ide-gtk.h"
 #include "ide-highlighter.h"
 #include "ide-highlight-engine.h"
 #include "ide-internal.h"
@@ -141,13 +142,39 @@ gboolean
 ide_buffer_get_has_diagnostics (IdeBuffer *self)
 {
   IdeBufferPrivate *priv = ide_buffer_get_instance_private (self);
+  guint size;
+  guint i;
 
   g_return_val_if_fail (IDE_IS_BUFFER (self), FALSE);
 
   if (priv->diagnostics == NULL)
     return FALSE;
 
-  return (ide_diagnostics_get_size (priv->diagnostics) > 0);
+  /*
+   * The diagnostics set might include warnings for files other than
+   * our own. So we need to verify they are for this file. As long as
+   * this is usually just used via bindings, its not expensive enough
+   * to matter much.
+   */
+
+  size = ide_diagnostics_get_size (priv->diagnostics);
+
+  for (i = 0; i < size; i++)
+    {
+      IdeDiagnostic *diag = ide_diagnostics_index (priv->diagnostics, i);
+      IdeSourceLocation *loc = ide_diagnostic_get_location (diag);
+      IdeFile *file;
+
+      if (loc == NULL)
+        continue;
+
+      file = ide_source_location_get_file (loc);
+
+      if (priv->file && file && ide_file_equal (priv->file, file))
+        return TRUE;
+    }
+
+  return FALSE;
 }
 
 /**
@@ -260,6 +287,8 @@ ide_buffer_clear_diagnostics (IdeBuffer *self)
 {
   IdeBufferPrivate *priv = ide_buffer_get_instance_private (self);
   GtkTextBuffer *buffer = (GtkTextBuffer *)self;
+  GtkTextTagTable *table;
+  GtkTextTag *tag;
   GtkTextIter begin;
   GtkTextIter end;
 
@@ -270,10 +299,19 @@ ide_buffer_clear_diagnostics (IdeBuffer *self)
 
   gtk_text_buffer_get_bounds (buffer, &begin, &end);
 
-  gtk_text_buffer_remove_tag_by_name (buffer, TAG_NOTE, &begin, &end);
-  gtk_text_buffer_remove_tag_by_name (buffer, TAG_WARNING, &begin, &end);
-  gtk_text_buffer_remove_tag_by_name (buffer, TAG_DEPRECATED, &begin, &end);
-  gtk_text_buffer_remove_tag_by_name (buffer, TAG_ERROR, &begin, &end);
+  table = gtk_text_buffer_get_tag_table (buffer);
+
+  if (NULL != (tag = gtk_text_tag_table_lookup (table, TAG_NOTE)))
+    ide_gtk_text_buffer_remove_tag (buffer, tag, &begin, &end, TRUE);
+
+  if (NULL != (tag = gtk_text_tag_table_lookup (table, TAG_WARNING)))
+    ide_gtk_text_buffer_remove_tag (buffer, tag, &begin, &end, TRUE);
+
+  if (NULL != (tag = gtk_text_tag_table_lookup (table, TAG_DEPRECATED)))
+    ide_gtk_text_buffer_remove_tag (buffer, tag, &begin, &end, TRUE);
+
+  if (NULL != (tag = gtk_text_tag_table_lookup (table, TAG_ERROR)))
+    ide_gtk_text_buffer_remove_tag (buffer, tag, &begin, &end, TRUE);
 }
 
 static void
@@ -2227,8 +2265,16 @@ ide_buffer_rehighlight (IdeBuffer *self)
 
   g_return_if_fail (IDE_IS_BUFFER (self));
 
-  if (priv->highlight_engine != NULL)
-    ide_highlight_engine_rebuild (priv->highlight_engine);
+  if (gtk_source_buffer_get_highlight_syntax (GTK_SOURCE_BUFFER (self)))
+    {
+      if (priv->highlight_engine != NULL)
+        {
+          ide_highlight_engine_rebuild (priv->highlight_engine);
+          IDE_EXIT;
+        }
+    }
+
+  ide_highlight_engine_clear (priv->highlight_engine);
 
   IDE_EXIT;
 }
