@@ -18,6 +18,8 @@
 
 #define G_LOG_DOMAIN "ide-workbench"
 
+#include <glib/gi18n.h>
+
 #include "ide-debug.h"
 #include "ide-genesis-perspective.h"
 #include "ide-greeter-perspective.h"
@@ -223,6 +225,8 @@ ide_workbench_constructed (GObject *object)
 
   G_OBJECT_CLASS (ide_workbench_parent_class)->constructed (object);
 
+  gtk_application_window_set_show_menubar (GTK_APPLICATION_WINDOW (self), FALSE);
+
   ide_workbench_actions_init (self);
 }
 
@@ -544,18 +548,44 @@ stablize_cb (gpointer data)
   return G_SOURCE_REMOVE;
 }
 
+static gboolean
+transform_title (GBinding     *binding,
+                 const GValue *from_value,
+                 GValue       *to_value,
+                 gpointer      user_data)
+{
+  const gchar *name = g_value_get_string (from_value);
+
+  if (name != NULL)
+    g_value_take_string (to_value, g_strdup_printf (_("%s - Builder"), name));
+  else
+    g_value_set_static_string (to_value, _("Builder"));
+
+  return TRUE;
+}
+
 void
 ide_workbench_set_context (IdeWorkbench *self,
                            IdeContext   *context)
 {
+  g_autoptr(GSettings) settings = NULL;
+  IdeProject *project;
   guint duration;
 
   g_return_if_fail (IDE_IS_WORKBENCH (self));
   g_return_if_fail (IDE_IS_CONTEXT (context));
   g_return_if_fail (self->context == NULL);
 
+  settings = g_settings_new ("org.gnome.builder");
+
   g_set_object (&self->context, context);
 
+  project = ide_context_get_project (context);
+  g_object_bind_property_full (project, "name",
+                               self, "title",
+                               G_BINDING_SYNC_CREATE,
+                               transform_title, NULL, NULL, NULL);
+
   self->addins = peas_extension_set_new (peas_engine_get_default (),
                                          IDE_TYPE_WORKBENCH_ADDIN,
                                          NULL);
@@ -585,8 +615,11 @@ ide_workbench_set_context (IdeWorkbench *self,
    * the stack transition results in non-smooth transitions. So instead,
    * we will delay until the transition has completed.
    */
-  duration = gtk_stack_get_transition_duration (self->top_stack);
-  g_timeout_add (STABLIZE_DELAY_MSEC + duration, restore_in_timeout, g_object_ref (context));
+  if (g_settings_get_boolean (settings, "restore-previous-files"))
+    {
+      duration = gtk_stack_get_transition_duration (self->top_stack);
+      g_timeout_add (STABLIZE_DELAY_MSEC + duration, restore_in_timeout, g_object_ref (context));
+    }
 }
 
 void
@@ -700,6 +733,23 @@ ide_workbench_get_visible_perspective (IdeWorkbench *self)
   return IDE_PERSPECTIVE (gtk_stack_get_visible_child (self->perspectives_stack));
 }
 
+static gboolean
+remove_early_perspectives (gpointer data)
+{
+  g_autoptr(IdeWorkbench) self = data;
+  GtkWidget *widget;
+
+  g_assert (IDE_IS_WORKBENCH (self));
+
+  widget = gtk_stack_get_child_by_name (self->top_stack, "greeter");
+  gtk_widget_destroy (widget);
+
+  widget = gtk_stack_get_child_by_name (self->top_stack, "genesis");
+  gtk_widget_destroy (widget);
+
+  return G_SOURCE_REMOVE;
+}
+
 void
 ide_workbench_set_visible_perspective (IdeWorkbench   *self,
                                        IdePerspective *perspective)
@@ -728,7 +778,12 @@ ide_workbench_set_visible_perspective (IdeWorkbench   *self,
 
   if ((stack == self->perspectives_stack) &&
       !ide_str_equal0 (gtk_stack_get_visible_child_name (self->top_stack), "perspectives"))
-    gtk_stack_set_visible_child_name (self->top_stack, "perspectives");
+    {
+      gtk_stack_set_visible_child_name (self->top_stack, "perspectives");
+      g_timeout_add (1000 + gtk_stack_get_transition_duration (self->top_stack),
+                     remove_early_perspectives,
+                     g_object_ref (self));
+    }
 }
 
 const gchar *
