@@ -24,6 +24,7 @@
 #include "ide-preferences-builtin.h"
 #include "ide-preferences-entry.h"
 #include "ide-preferences-language-row.h"
+#include "ide-preferences-spin-button.h"
 
 static void
 ide_preferences_builtin_register_plugins (IdePreferences *preferences)
@@ -99,9 +100,9 @@ ide_preferences_builtin_register_appearance (IdePreferences *preferences)
     }
 
   ide_preferences_add_list_group (preferences, "appearance", "font", _("Font"), 200);
-  ide_preferences_add_font_button (preferences, "appearance", "font", "org.gnome.builder.editor", "font-name", _("Editor"), _("editor font monospace"), 0);
+  ide_preferences_add_font_button (preferences, "appearance", "font", "org.gnome.builder.editor", "font-name", _("Editor"), C_("Keywords", "editor font monospace"), 0);
   /* XXX: This belongs in terminal addin */
-  ide_preferences_add_font_button (preferences, "appearance", "font", "org.gnome.builder.terminal", "font-name", _("Terminal"), _("terminal font monospace"), 0);
+  ide_preferences_add_font_button (preferences, "appearance", "font", "org.gnome.builder.terminal", "font-name", _("Terminal"), C_("Keywords", "terminal font monospace"), 0);
 }
 
 static void
@@ -237,13 +238,74 @@ ide_preferences_builtin_register_languages (IdePreferences *preferences)
   ide_preferences_add_radio (preferences, "languages.id", "indentation", "org.gnome.builder.editor.language", "auto-indent", "/org/gnome/builder/editor/language/{id}/", NULL, _("Automatically indent"), _("Indent source code as you type"), NULL, 30);
 }
 
+static gboolean
+workers_output (GtkSpinButton *button)
+{
+  GtkAdjustment *adj = gtk_spin_button_get_adjustment (button);
+
+  if (gtk_adjustment_get_value (adj) == -1)
+    {
+      gtk_entry_set_text (GTK_ENTRY (button), _("Default"));
+      return TRUE;
+    }
+  else if (gtk_adjustment_get_value (adj) == 0)
+    {
+      gtk_entry_set_text (GTK_ENTRY (button), _("Number of CPU"));
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
+static gint
+workers_input (GtkSpinButton *button,
+               gdouble       *new_value)
+{
+  const gchar *text = gtk_entry_get_text (GTK_ENTRY (button));
+
+  if (g_strcmp0 (text, _("Default")) == 0)
+    {
+      *new_value = -1;
+      return TRUE;
+    }
+  else if (g_strcmp0 (text, _("Number of CPU")) == 0)
+    {
+      *new_value = 0;
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
 static void
 ide_preferences_builtin_register_build (IdePreferences *preferences)
 {
+  GtkWidget *widget, *bin;
+  guint id;
+
   ide_preferences_add_page (preferences, "build", _("Build"), 500);
 
   ide_preferences_add_list_group (preferences, "build", "basic", _("General"), 0);
-  ide_preferences_add_spin_button (preferences, "build", "basic", "org.gnome.builder.build", "parallel", "/org/gnome/builder/build/", _("Build Workers"), _("Number of parallel build workers"), NULL, 0);
+  id = ide_preferences_add_spin_button (preferences, "build", "basic", "org.gnome.builder.build", "parallel", "/org/gnome/builder/build/", _("Build Workers"), _("Number of parallel build workers"), NULL, 0);
+
+  bin = ide_preferences_get_widget (preferences, id);
+  widget = ide_preferences_spin_button_get_spin_button (IDE_PREFERENCES_SPIN_BUTTON (bin));
+  gtk_entry_set_width_chars (GTK_ENTRY (widget), 20);
+  g_signal_connect (widget, "input", G_CALLBACK (workers_input), NULL);
+  g_signal_connect (widget, "output", G_CALLBACK (workers_output), NULL);
+}
+
+static void
+ide_preferences_builtin_register_projects (IdePreferences *preferences)
+{
+  ide_preferences_add_page (preferences, "projects", _("Projects"), 450);
+
+  ide_preferences_add_list_group (preferences, "projects", "directory", _("Workspace"), 0);
+  ide_preferences_add_file_chooser (preferences, "projects", "directory", "org.gnome.builder", "projects-directory", NULL, _("Projects directory"), _("A place for all your projects"), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, NULL, 0);
+  ide_preferences_add_switch (preferences, "projects", "directory", "org.gnome.builder", "restore-previous-files", NULL, NULL, _("Restore previously opened files"), _("Open previously opened files when loading a project"), NULL, 10);
+
+  ide_preferences_add_list_group (preferences, "projects", "discovery", _("Project Discovery"), 0);
+  ide_preferences_add_switch (preferences, "projects", "discovery", "org.gnome.builder", "enable-project-miners", NULL, NULL, _("Discover projects on my computer"), _("Scan your computer for existing projects"), NULL, 0);
 }
 
 void
@@ -257,4 +319,5 @@ _ide_preferences_builtin_register (IdePreferences *preferences)
   ide_preferences_builtin_register_keyboard (preferences);
   ide_preferences_builtin_register_plugins (preferences);
   ide_preferences_builtin_register_build (preferences);
+  ide_preferences_builtin_register_projects (preferences);
 }
