@@ -28,9 +28,12 @@
 
 #include "ide-autotools-build-task.h"
 
+#define FLAG_SET(_f,_n) (((_f) & (_n)) != 0)
+#define FLAG_UNSET(_f,_n) (((_f) & (_n)) == 0)
+
 struct _IdeAutotoolsBuildTask
 {
-  IdeObject         parent_instance;
+  IdeBuildResult    parent;
   IdeConfiguration *configuration;
   GFile            *directory;
   GPtrArray        *extra_targets;
@@ -48,6 +51,7 @@ typedef struct
   gchar      **configure_argv;
   gchar      **make_targets;
   IdeRuntime  *runtime;
+  guint        sequence;
   guint        require_autogen : 1;
   guint        require_configure : 1;
   guint        bootstrap_only : 1;
@@ -418,8 +422,9 @@ worker_state_new (IdeAutotoolsBuildTask *self,
     project_dir = g_object_ref (project_file);
 
   state = g_slice_new0 (WorkerState);
-  state->require_autogen = self->require_autogen || !!(flags & IDE_BUILDER_BUILD_FLAGS_FORCE_BOOTSTRAP);
-  state->require_configure = self->require_configure || (state->require_autogen && !(flags & IDE_BUILDER_BUILD_FLAGS_NO_CONFIGURE));
+  state->sequence = ide_configuration_get_sequence (self->configuration);
+  state->require_autogen = self->require_autogen || FLAG_SET (flags, IDE_BUILDER_BUILD_FLAGS_FORCE_BOOTSTRAP);
+  state->require_configure = self->require_configure || (state->require_autogen && FLAG_UNSET (flags, IDE_BUILDER_BUILD_FLAGS_NO_CONFIGURE));
   state->directory_path = g_file_get_path (self->directory);
   state->project_path = g_file_get_path (project_dir);
   state->system_type = g_strdup (ide_device_get_system_type (device));
@@ -436,21 +441,24 @@ worker_state_new (IdeAutotoolsBuildTask *self,
 
   make_targets = g_ptr_array_new ();
 
-  if (0 != (flags & IDE_BUILDER_BUILD_FLAGS_FORCE_CLEAN))
+  if (FLAG_SET (flags, IDE_BUILDER_BUILD_FLAGS_FORCE_CLEAN))
     {
-      state->require_autogen = TRUE;
-      state->require_configure = TRUE;
+      if (FLAG_UNSET (flags, IDE_BUILDER_BUILD_FLAGS_NO_BUILD))
+        {
+          state->require_autogen = TRUE;
+          state->require_configure = TRUE;
+        }
       g_ptr_array_add (make_targets, g_strdup ("clean"));
     }
 
-  if (0 == (flags & IDE_BUILDER_BUILD_FLAGS_NO_BUILD))
+  if (FLAG_UNSET (flags, IDE_BUILDER_BUILD_FLAGS_NO_BUILD))
     g_ptr_array_add (make_targets, g_strdup ("all"));
 
   g_ptr_array_add (make_targets, NULL);
 
   state->make_targets = (gchar **)g_ptr_array_free (make_targets, FALSE);
 
-  if (0 != (flags & IDE_BUILDER_BUILD_FLAGS_NO_CONFIGURE))
+  if (FLAG_SET (flags, IDE_BUILDER_BUILD_FLAGS_NO_CONFIGURE))
     {
       state->require_autogen = TRUE;
       state->require_configure = TRUE;
@@ -569,11 +577,18 @@ ide_autotools_build_task_execute_finish (IdeAutotoolsBuildTask  *self,
                                          GError                **error)
 {
   GTask *task = (GTask *)result;
+  WorkerState *state;
+  guint sequence;
 
   g_return_val_if_fail (IDE_IS_AUTOTOOLS_BUILD_TASK (self), FALSE);
-  g_return_val_if_fail (G_IS_TASK (result), FALSE);
   g_return_val_if_fail (G_IS_TASK (task), FALSE);
 
+  state = g_task_get_task_data (G_TASK (task));
+  sequence = ide_configuration_get_sequence (self->configuration);
+
+  if ((state != NULL) &&  (state->sequence == sequence))
+    ide_configuration_set_dirty (self->configuration, FALSE);
+
   return g_task_propagate_boolean (task, error);
 }
 
