@@ -39,11 +39,12 @@ struct _IdeCIndenter
   /* no reference */
   IdeSourceView *view;
 
-  gint           scope_indent;
+  gint           pre_scope_indent;
+  gint           post_scope_indent;
   gint           condition_indent;
   gint           directive_indent;
-
-  guint          space_before_paren : 1;
+  gint           extra_label_indent;
+  gint           case_indent;
 };
 
 static void indenter_iface_init (IdeIndenterInterface *iface);
@@ -161,34 +162,50 @@ line_is_whitespace_until (GtkTextIter *iter)
   return TRUE;
 }
 
-static gboolean
-backward_find_keyword (GtkTextIter *iter,
-                       const gchar *keyword,
-                       GtkTextIter *limit)
+static gboolean is_special (const GtkTextIter *iter)
 {
-  GtkTextIter begin;
-  GtkTextIter end;
+  GtkSourceBuffer *buffer;
 
-  /*
-   * If we find the keyword, check to see that the character before it
-   * is either a newline or some other space character. (ie, not part of a
-   * function name like foo_do().
-   */
-  if (gtk_text_iter_backward_search (iter, keyword, GTK_TEXT_SEARCH_TEXT_ONLY,
-                                     &begin, &end, limit))
+  buffer = GTK_SOURCE_BUFFER (gtk_text_iter_get_buffer (iter));
+  return (gtk_source_buffer_iter_has_context_class (buffer, iter, "string") ||
+          gtk_source_buffer_iter_has_context_class (buffer, iter, "comment"));
+}
+
+static gboolean
+backward_find_keyword (GtkTextIter       *iter,
+                       const gchar       *keyword,
+                       const GtkTextIter *limit)
+{
+  GtkTextIter copy = *iter;
+
+  while (gtk_text_iter_compare (&copy, limit) > 0)
     {
-      GtkTextIter copy;
-      gunichar ch;
+      GtkTextIter begin;
+      GtkTextIter end;
 
-      gtk_text_iter_assign (&copy, &begin);
+      if (!gtk_text_iter_backward_search (&copy, keyword, GTK_TEXT_SEARCH_TEXT_ONLY, &begin, &end, limit))
+        return FALSE;
 
-      if (!gtk_text_iter_backward_char (&copy) ||
-          !(ch = gtk_text_iter_get_char (&copy)) ||
-          g_unichar_isspace (ch))
+      if (!is_special (&begin) &&
+          gtk_text_iter_starts_word (&begin) &&
+          gtk_text_iter_ends_word (&end))
         {
-          gtk_text_iter_assign (iter, &begin);
+          GtkTextIter prev = begin;
+          gunichar ch;
+
+          if (gtk_text_iter_backward_char (&prev) &&
+              (ch = gtk_text_iter_get_char (&prev)) &&
+              !g_unichar_isspace (ch))
+            {
+              goto again;
+            }
+
+          *iter = begin;
           return TRUE;
         }
+
+    again:
+      copy = begin;
     }
 
   return FALSE;
@@ -490,6 +507,9 @@ in_comment (const GtkTextIter *location,
   IDE_RETURN (TRUE);
 }
 
+#define GET_LINE_OFFSET(_iter) \
+  gtk_source_view_get_visual_column(GTK_SOURCE_VIEW(view), _iter)
+
 static gchar *
 c_indenter_indent (IdeCIndenter  *c,
                    GtkTextView   *view,
@@ -533,7 +553,7 @@ c_indenter_indent (IdeCIndenter  *c,
       guint offset;
 
       gtk_text_iter_assign (iter, &match_begin);
-      offset = gtk_text_iter_get_line_offset (iter);
+      offset = GET_LINE_OFFSET (iter);
       if (comment_type == COMMENT_C89)
         {
           build_indent (c, offset + 1, iter, str);
@@ -583,7 +603,7 @@ c_indenter_indent (IdeCIndenter  *c,
           !backward_find_matching_char (iter, '}'))
         IDE_GOTO (cleanup);
 
-      offset = gtk_text_iter_get_line_offset (iter);
+      offset = GET_LINE_OFFSET (iter);
 
       if (gtk_text_iter_get_char (iter) == '(')
         offset++;
@@ -594,8 +614,8 @@ c_indenter_indent (IdeCIndenter  *c,
            * like "enum {".
            */
           if (backward_to_line_first_char (iter))
-            offset = gtk_text_iter_get_line_offset (iter);
-          offset += c->scope_indent;
+            offset = GET_LINE_OFFSET (iter);
+          offset += c->post_scope_indent;
         }
 
       build_indent (c, offset, iter, str);
@@ -612,7 +632,7 @@ c_indenter_indent (IdeCIndenter  *c,
 
       if (backward_find_stmt_expr (iter))
         {
-          offset = gtk_text_iter_get_line_offset (iter);
+          offset = GET_LINE_OFFSET (iter);
           build_indent (c, offset, iter, str);
           IDE_GOTO (cleanup);
         }
@@ -628,7 +648,7 @@ c_indenter_indent (IdeCIndenter  *c,
     {
       guint offset;
 
-      offset = gtk_text_iter_get_line_offset (iter);
+      offset = GET_LINE_OFFSET (iter);
       build_indent (c, offset + 1, iter, str);
       IDE_GOTO (cleanup);
     }
@@ -645,12 +665,12 @@ c_indenter_indent (IdeCIndenter  *c,
 
       if (gtk_text_iter_forward_char (iter))
         {
-          guint offset = gtk_text_iter_get_line_offset (iter) - 1;
+          guint offset = GET_LINE_OFFSET (iter) - 1;
 
           if (backward_find_matching_char (iter, '}'))
             {
-              offset = gtk_text_iter_get_line_offset (iter);
-              offset += c->scope_indent;
+              offset = GET_LINE_OFFSET (iter);
+              offset += c->post_scope_indent;
             }
 
           build_indent (c, offset, iter, str);
@@ -672,7 +692,7 @@ c_indenter_indent (IdeCIndenter  *c,
       if (backward_find_matching_char (iter, ')') &&
           backward_find_condition_keyword (iter))
         {
-          guint offset = gtk_text_iter_get_line_offset (iter);
+          guint offset = GET_LINE_OFFSET (iter);
           build_indent (c, offset + c->condition_indent, iter, str);
           IDE_GOTO (cleanup);
         }
@@ -694,8 +714,8 @@ c_indenter_indent (IdeCIndenter  *c,
       if (!line_is_whitespace_until (&match_begin))
         backward_to_line_first_char (&match_begin);
 
-      offset = gtk_text_iter_get_line_offset (&match_begin);
-      build_indent (c, offset + c->scope_indent, iter, str);
+      offset = GET_LINE_OFFSET (&match_begin);
+      build_indent (c, offset + c->pre_scope_indent, iter, str);
       IDE_GOTO (cleanup);
     }
 
@@ -709,8 +729,8 @@ c_indenter_indent (IdeCIndenter  *c,
         {
           guint offset;
 
-          offset = gtk_text_iter_get_line_offset (iter);
-          build_indent (c, offset + c->scope_indent, iter, str);
+          offset = GET_LINE_OFFSET (iter);
+          build_indent (c, offset + c->post_scope_indent, iter, str);
           IDE_GOTO (cleanup);
         }
       else
@@ -719,8 +739,8 @@ c_indenter_indent (IdeCIndenter  *c,
             {
               guint offset;
 
-              offset = gtk_text_iter_get_line_offset (iter);
-              build_indent (c, offset + c->scope_indent, iter, str);
+              offset = GET_LINE_OFFSET (iter);
+              build_indent (c, offset + c->post_scope_indent, iter, str);
               IDE_GOTO (cleanup);
             }
         }
@@ -771,9 +791,71 @@ maybe_close_comment (IdeCIndenter *c,
 }
 
 static gchar *
-maybe_unindent_brace (IdeCIndenter *c,
-                      GtkTextIter           *begin,
-                      GtkTextIter           *end)
+maybe_unindent_opening_brace (IdeCIndenter *c,
+                              GtkTextView  *view,
+                              GtkTextIter  *begin,
+                              GtkTextIter  *end,
+                              gint         *cursor_offset)
+{
+  GtkTextIter copy;
+
+  g_assert (IDE_IS_C_INDENTER (c));
+  g_assert (begin);
+  g_assert (end);
+
+  copy = *begin;
+
+  /*
+   * Make sure we just inserted a { and then move before it.
+   * Ensure that we only have whitespace before the {.
+   */
+  if (!gtk_text_iter_backward_char (&copy) ||
+      ('{' != gtk_text_iter_get_char (&copy)) ||
+      !gtk_text_iter_backward_char (&copy))
+    return NULL;
+
+  /*
+   * Find the opening of the parent scope.
+   * We should be at that + post_scope_indent, which is where
+   * our conditional would have started.
+   */
+  if (line_is_whitespace_until (&copy) && backward_find_matching_char (&copy, '}'))
+    {
+      guint offset;
+      GString *str;
+
+      backward_to_line_first_char (&copy);
+
+      offset = GET_LINE_OFFSET (&copy);
+      str = g_string_new (NULL);
+      build_indent (c, offset + c->post_scope_indent + c->pre_scope_indent, &copy, str);
+      g_string_append_c (str, '{');
+
+      if (ide_source_view_get_insert_matching_brace (IDE_SOURCE_VIEW (view)))
+        {
+          g_string_append_c (str, '}');
+          *cursor_offset = -1;
+        }
+
+      gtk_text_iter_set_line_offset (begin, 0);
+
+      return g_string_free (str, FALSE);
+    }
+
+  if (ide_source_view_get_insert_matching_brace (IDE_SOURCE_VIEW (view)))
+    {
+      *cursor_offset = -1;
+      return g_strdup ("}");
+    }
+
+  return NULL;
+}
+
+static gchar *
+maybe_unindent_closing_brace (IdeCIndenter *c,
+                              GtkTextView  *view,
+                              GtkTextIter  *begin,
+                              GtkTextIter  *end)
 {
   GtkTextIter saved;
   gchar *ret = NULL;
@@ -803,7 +885,7 @@ maybe_unindent_brace (IdeCIndenter *c,
       if (!starts_line_space_ok (begin))
         backward_to_line_first_char (begin);
 
-      offset = gtk_text_iter_get_line_offset (begin);
+      offset = GET_LINE_OFFSET (begin);
       str = g_string_new (NULL);
       build_indent (c, offset, begin, str);
       g_string_append_c (str, '}');
@@ -828,8 +910,8 @@ maybe_unindent_brace (IdeCIndenter *c,
 
 static gchar *
 maybe_unindent_hash (IdeCIndenter *c,
-                     GtkTextIter           *begin,
-                     GtkTextIter           *end)
+                     GtkTextIter  *begin,
+                     GtkTextIter  *end)
 {
   GtkTextIter saved;
   gchar *ret = NULL;
@@ -885,53 +967,6 @@ line_starts_with_fuzzy (const GtkTextIter *iter,
   return ret;
 }
 
-#if 0
-static gchar *
-maybe_space_before_paren (IdeCIndenter *c,
-                          GtkTextIter           *begin,
-                          GtkTextIter           *end)
-{
-  GtkTextIter match_begin;
-  GtkTextIter copy;
-  gunichar ch;
-
-  g_return_val_if_fail (IDE_IS_C_INDENTER (c), NULL);
-  g_return_val_if_fail (begin, NULL);
-  g_return_val_if_fail (end, NULL);
-
-  if (!c->priv->space_before_paren)
-    return NULL;
-
-  if (in_c89_comment (begin, &match_begin))
-    return NULL;
-
-  /* ignore preprocessor #define */
-  if (line_starts_with_fuzzy (begin, "#"))
-    return NULL;
-
-  gtk_text_iter_assign (&copy, begin);
-
-  /*
-   * Move back to the character just inserted.
-   */
-  if (gtk_text_iter_backward_char (begin) &&
-      (ch = gtk_text_iter_get_char (begin)) &&
-      (ch == '(') &&
-      gtk_text_iter_backward_char (begin) &&
-      (ch = gtk_text_iter_get_char (begin)) &&
-      !g_unichar_isspace (ch) &&
-      g_unichar_isalnum (ch))
-    {
-      gtk_text_iter_forward_char (begin);
-      return g_strdup (" (");
-    }
-
-  gtk_text_iter_assign (begin, &copy);
-
-  return NULL;
-}
-#endif
-
 static gchar *
 format_parameter (const Parameter *param,
                   guint            max_type,
@@ -1024,8 +1059,8 @@ format_parameters (GtkTextIter *begin,
 
 static gchar *
 maybe_align_parameters (IdeCIndenter *c,
-                        GtkTextIter           *begin,
-                        GtkTextIter           *end)
+                        GtkTextIter  *begin,
+                        GtkTextIter  *end)
 {
   GtkTextIter match_begin;
   GtkTextIter copy;
@@ -1068,55 +1103,6 @@ maybe_align_parameters (IdeCIndenter *c,
   IDE_RETURN (ret);
 }
 
-#if 0
-static gchar *
-maybe_add_brace (IdeCIndenter *c,
-                 GtkTextIter           *begin,
-                 GtkTextIter           *end,
-                 gint                  *cursor_offset)
-{
-  GtkTextIter iter;
-
-  gtk_text_iter_assign (&iter, begin);
-
-  if (gtk_text_iter_backward_char (&iter) &&
-      (gtk_text_iter_get_char (&iter) == '{') &&
-      (gtk_text_iter_get_char (begin) != '}'))
-    {
-      GtkTextIter copy = iter;
-
-      gtk_text_iter_assign (&copy, &iter);
-
-      if (gtk_text_iter_backward_word_start (&copy))
-        {
-          GtkTextIter copy2 = copy;
-
-          if (gtk_text_iter_forward_word_end (&copy2))
-            {
-              gchar *word;
-
-              word = gtk_text_iter_get_slice (&copy, &copy2);
-
-              if ((g_strcmp0 (word, "enum") == 0) ||
-                  (g_strcmp0 (word, "struct") == 0))
-                {
-                  *cursor_offset = -2;
-                  g_free (word);
-                  return g_strdup ("};");
-                }
-
-              g_free (word);
-            }
-        }
-
-      *cursor_offset = -1;
-      return g_strdup ("}");
-    }
-
-  return NULL;
-}
-#endif
-
 static gboolean
 line_is_case (const GtkTextIter *line)
 {
@@ -1201,8 +1187,9 @@ line_is_label (const GtkTextIter *line)
 
 static gchar *
 maybe_unindent_case_label (IdeCIndenter *c,
-                           GtkTextIter           *begin,
-                           GtkTextIter           *end)
+                           GtkTextView  *view,
+                           GtkTextIter  *begin,
+                           GtkTextIter  *end)
 {
   GtkTextIter match_begin;
   GtkTextIter iter;
@@ -1221,38 +1208,27 @@ maybe_unindent_case_label (IdeCIndenter *c,
     {
       if (backward_find_matching_char (&iter, '}'))
         {
-          if (line_is_whitespace_until (&iter))
-            {
-              GString *str;
-              guint offset;
+          GString *str;
+          guint offset;
 
-              str = g_string_new (NULL);
-              offset = gtk_text_iter_get_line_offset (&iter);
-              build_indent (c, offset, &iter, str);
-              while (!gtk_text_iter_starts_line (begin))
-                gtk_text_iter_backward_char (begin);
-              gtk_text_iter_assign (end, begin);
-              while (g_unichar_isspace (gtk_text_iter_get_char (end)))
-                if (!gtk_text_iter_forward_char (end))
-                  IDE_RETURN (NULL);
-              return g_string_free (str, FALSE);
-            }
-          else
-            {
-              if (backward_to_line_first_char (&iter))
-                {
-#if 0
-                  TODO ("Deal with nested {");
-#endif
-                }
-            }
+          if (!line_is_whitespace_until (&iter))
+            backward_to_line_first_char (&iter);
+
+          str = g_string_new (NULL);
+          offset = GET_LINE_OFFSET (&iter);
+          build_indent (c, offset + c->case_indent, &iter, str);
+          while (!gtk_text_iter_starts_line (begin))
+            gtk_text_iter_backward_char (begin);
+          gtk_text_iter_assign (end, begin);
+          while (g_unichar_isspace (gtk_text_iter_get_char (end)))
+            if (!gtk_text_iter_forward_char (end))
+              IDE_RETURN (NULL);
+          return g_string_free (str, FALSE);
         }
     }
   else if (line_is_label (&iter))
     {
-#if 0
-      TODO ("allow configurable label indent");
-#endif
+      GString *str = g_string_new (NULL);
 
       ITER_INIT_LINE_START (begin, &iter);
       ITER_INIT_LINE_START (end, &iter);
@@ -1261,7 +1237,11 @@ maybe_unindent_case_label (IdeCIndenter *c,
         if (!gtk_text_iter_forward_char (end))
           return NULL;
 
-      return g_strdup ("");
+      if (c->extra_label_indent)
+        for (int i = 0; i < c->extra_label_indent; i++)
+          g_string_append_c (str, ' ');
+
+      return g_string_free (str, FALSE);
     }
 
   IDE_RETURN (NULL);
@@ -1279,6 +1259,7 @@ ide_c_indenter_is_trigger (IdeIndenter *indenter,
         return FALSE;
       /* Fall through */
 
+    case GDK_KEY_braceleft:
     case GDK_KEY_braceright:
     case GDK_KEY_colon:
     case GDK_KEY_numbersign:
@@ -1335,9 +1316,9 @@ ide_c_indenter_format (IdeIndenter    *indenter,
         if (backward_find_matching_char (&iter, '}'))
           {
             if (line_is_whitespace_until (&iter))
-              offset = gtk_text_iter_get_line_offset (&iter);
+              offset = GET_LINE_OFFSET (&iter);
             else if (backward_to_line_first_char (&iter))
-              offset = gtk_text_iter_get_line_offset (&iter);
+              offset = GET_LINE_OFFSET (&iter);
             build_indent (c, offset, &iter, str);
             g_string_prepend (str, "\n");
             g_string_prepend (str, ret);
@@ -1353,17 +1334,26 @@ ide_c_indenter_format (IdeIndenter    *indenter,
 
   case GDK_KEY_braceright:
     /*
-     * Probably need to unindent this line.
-     * TODO: Maybe overwrite character.
+     * Possibly need to unindent this line.
      */
-    ret = maybe_unindent_brace (c, begin, end);
+    ret = maybe_unindent_closing_brace (c, view, begin, end);
+    break;
+
+  case GDK_KEY_braceleft:
+    /*
+     * Maybe unindent the opening brace to match the conditional.
+     * This could happen if we are doing k&r/linux/etc where the open
+     * brace has less indentation than the natural single line conditional
+     * child statement.
+     */
+    ret = maybe_unindent_opening_brace (c, view, begin, end, cursor_offset);
     break;
 
   case GDK_KEY_colon:
     /*
      * If this is a label or a case, adjust indentation.
      */
-    ret = maybe_unindent_case_label (c, begin, end);
+    ret = maybe_unindent_case_label (c, view, begin, end);
     break;
 
   case GDK_KEY_numbersign:
@@ -1417,9 +1407,10 @@ static void
 ide_c_indenter_init (IdeCIndenter *self)
 {
   self->condition_indent = 2;
-  self->scope_indent = 2;
+  self->pre_scope_indent = 2;
+  self->post_scope_indent = 2;
   self->directive_indent = G_MININT;
-  self->space_before_paren = TRUE;
+  self->case_indent = 0;
 }
 
 void
