@@ -37,15 +37,16 @@ struct _IdeGitCloneWidget
   GtkEntry             *clone_uri_entry;
   GtkLabel             *clone_error_label;
   GtkProgressBar       *clone_progress;
+  GtkSpinner           *clone_spinner;
 
   guint                 is_ready : 1;
 };
 
 typedef struct
 {
-  gchar *uri;
-  GFile *location;
-  GFile *project_file;
+  IdeVcsUri *uri;
+  GFile     *location;
+  GFile     *project_file;
 } CloneRequest;
 
 enum {
@@ -63,7 +64,7 @@ clone_request_free (gpointer data)
 
   if (req != NULL)
     {
-      g_clear_pointer (&req->uri, g_free);
+      g_clear_pointer (&req->uri, ide_vcs_uri_unref);
       g_clear_object (&req->location);
       g_clear_object (&req->project_file);
       g_slice_free (CloneRequest, req);
@@ -71,8 +72,8 @@ clone_request_free (gpointer data)
 }
 
 static CloneRequest *
-clone_request_new (const gchar *uri,
-                   GFile       *location)
+clone_request_new (IdeVcsUri *uri,
+                   GFile     *location)
 {
   CloneRequest *req;
 
@@ -80,7 +81,7 @@ clone_request_new (const gchar *uri,
   g_assert (location);
 
   req = g_slice_new0 (CloneRequest);
-  req->uri = g_strdup (uri);
+  req->uri = ide_vcs_uri_ref (uri);
   req->location = g_object_ref (location);
   req->project_file = NULL;
 
@@ -107,8 +108,8 @@ ide_git_clone_widget_uri_changed (IdeGitCloneWidget *self,
       gchar *name = NULL;
 
       g_object_set (self->clone_uri_entry,
+                    "secondary-icon-tooltip-text", "",
                     "secondary-icon-name", NULL,
-                    "secondary-icon-tooltip-text", NULL,
                     NULL);
 
       path = ide_vcs_uri_get_path (uri);
@@ -188,6 +189,7 @@ ide_git_clone_widget_class_init (IdeGitCloneWidgetClass *klass)
   gtk_widget_class_bind_template_child (widget_class, IdeGitCloneWidget, clone_location_button);
   gtk_widget_class_bind_template_child (widget_class, IdeGitCloneWidget, clone_location_entry);
   gtk_widget_class_bind_template_child (widget_class, IdeGitCloneWidget, clone_progress);
+  gtk_widget_class_bind_template_child (widget_class, IdeGitCloneWidget, clone_spinner);
   gtk_widget_class_bind_template_child (widget_class, IdeGitCloneWidget, clone_uri_entry);
 }
 
@@ -205,7 +207,11 @@ ide_git_clone_widget_init (IdeGitCloneWidget *self)
 
   if (!ide_str_empty0 (path))
     {
-      projects_dir = g_build_filename (g_get_home_dir (), path, NULL);
+      if (!g_path_is_absolute (path))
+        projects_dir = g_build_filename (g_get_home_dir (), path, NULL);
+      else
+        projects_dir = g_steal_pointer (&path);
+
       gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (self->clone_location_button),
                                        projects_dir);
     }
@@ -290,6 +296,7 @@ ide_git_clone_widget_worker (GTask        *task,
                              gpointer      task_data,
                              GCancellable *cancellable)
 {
+  g_autofree gchar *uristr = NULL;
   IdeGitCloneWidget *self = source_object;
   GgitRepository *repository;
   CloneRequest *req = task_data;
@@ -317,7 +324,9 @@ ide_git_clone_widget_worker (GTask        *task,
   ggit_clone_options_set_fetch_options (clone_options, fetch_options);
   g_clear_pointer (&fetch_options, ggit_fetch_options_free);
 
-  repository = ggit_repository_clone (req->uri, req->location, clone_options, &error);
+  uristr = ide_vcs_uri_to_string (req->uri);
+
+  repository = ggit_repository_clone (uristr, req->location, clone_options, &error);
 
   g_clear_object (&callbacks);
   g_clear_object (&clone_options);
@@ -343,19 +352,33 @@ ide_git_clone_widget_clone_async (IdeGitCloneWidget   *self,
   g_autoptr(GTask) task = NULL;
   g_autoptr(GFile) location = NULL;
   g_autoptr(GFile) child = NULL;
+  g_autoptr(IdeVcsUri) uri = NULL;
   CloneRequest *req;
-  const gchar *uri;
+  const gchar *uristr;
   const gchar *child_name;
 
   g_return_if_fail (IDE_IS_GIT_CLONE_WIDGET (self));
   g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
 
+  task = g_task_new (self, cancellable, callback, user_data);
+
   gtk_label_set_label (self->clone_error_label, NULL);
 
-  uri = gtk_entry_get_text (self->clone_uri_entry);
+  uristr = gtk_entry_get_text (self->clone_uri_entry);
   child_name = gtk_entry_get_text (self->clone_location_entry);
   location = gtk_file_chooser_get_file (GTK_FILE_CHOOSER (self->clone_location_button));
 
+  uri = ide_vcs_uri_new (uristr);
+
+  if (uri == NULL)
+    {
+      g_task_return_new_error (task,
+                               G_IO_ERROR,
+                               G_IO_ERROR_INVAL,
+                               _("A valid Git URL is required"));
+      return;
+    }
+
   if (child_name != NULL)
     {
       child = g_file_get_child (location, child_name);
@@ -366,7 +389,12 @@ ide_git_clone_widget_clone_async (IdeGitCloneWidget   *self,
       req = clone_request_new (uri, location);
     }
 
-  task = g_task_new (self, cancellable, callback, user_data);
+  gtk_spinner_start (self->clone_spinner);
+
+  gtk_widget_set_sensitive (GTK_WIDGET (self->clone_location_button), FALSE);
+  gtk_widget_set_sensitive (GTK_WIDGET (self->clone_location_entry), FALSE);
+  gtk_widget_set_sensitive (GTK_WIDGET (self->clone_uri_entry), FALSE);
+
   g_task_set_task_data (task, req, clone_request_free);
   g_task_run_in_thread (task, ide_git_clone_widget_worker);
 }
@@ -379,5 +407,11 @@ ide_git_clone_widget_clone_finish (IdeGitCloneWidget  *self,
   g_return_val_if_fail (IDE_IS_GIT_CLONE_WIDGET (self), FALSE);
   g_return_val_if_fail (G_IS_TASK (result), FALSE);
 
+  gtk_spinner_stop (self->clone_spinner);
+
+  gtk_widget_set_sensitive (GTK_WIDGET (self->clone_location_button), TRUE);
+  gtk_widget_set_sensitive (GTK_WIDGET (self->clone_location_entry), TRUE);
+  gtk_widget_set_sensitive (GTK_WIDGET (self->clone_uri_entry), TRUE);
+
   return g_task_propagate_boolean (G_TASK (result), error);
 }
